package edu.swe622.fers.service;

import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.Map;

import org.apache.axis2.AxisFault;

import edu.swe622.fers.model.Location;
import edu.swe622.fers.model.LocationMismatch;
import edu.swe622.fers.model.Sensor;
import edu.swe622.fers.util.ServiceUtil;

public class AmbulanceDispatchService extends VehicleDispatchService
{
    protected String serviceType = "A";  //  A(mbulance) or F(ire).
    
    
    /**
     *  Services
     */
    public void requestEngineDispatch (Sensor newSensor) 
    {       
        System.out.println("AmbulanceDispatchService.requestEngineDispatch: newSensor: " + newSensor.toString());
        
        Location newSchoolLocation = null;  // null make netbeans happy.
        int newPeopleCount = 300; // default is 300.
        
        Iterator incidentsIterator = this.incidents.iterator();
        
        Incident currentIncident = null;  //null makes netbeans happy.
        Incident nextIncedent;
        boolean foundMatchingIncident = false;
       
        if(!initialized){
        	this.dispatchPoolInit();
        	initialized = true;
        }
        
        // get school service URL from jUDDI    
//        Map schoolService = ServiceUtil.getServiceAccessPoints(newSensor.getSchoolId());    
//        String schoolServiceUrl = (String)schoolService.keySet().iterator().next();
        String serviceName = newSensor.getSchoolId();
        String schoolServiceUrl = "http://localhost:8080/"+serviceName.substring(0,serviceName.length()-1)+"WEB/services/"+serviceName;

        // get location and people count from school service
        try 
        {		                      
            // Get Service Stub for this School.
            SchoolServiceStub schoolStub = new SchoolServiceStub(schoolServiceUrl);
            newSchoolLocation = convertToLocation( schoolStub.getLocation().get_return() );
            newPeopleCount = schoolStub.numofpeople().get_return();
        }
        catch (AxisFault e) { e.printStackTrace(); } 
        catch (RemoteException e) { e.printStackTrace(); } 
    
        
        // See if request is for existing incident.
        while( incidentsIterator.hasNext() && !foundMatchingIncident )
        {
            nextIncedent = (Incident)incidentsIterator.next();
            if ( nextIncedent.getSchoolID().equals(newSensor.getSchoolId()) )
            {
                foundMatchingIncident = true;   
                currentIncident = nextIncedent;
                
                // Get updated people count
                try 
                {		                      
                    // Get Service Stub for this School.
                    SchoolServiceStub schoolStub = new SchoolServiceStub(schoolServiceUrl);
                    newPeopleCount = schoolStub.numofpeople().get_return();
                }
                catch (AxisFault e) { e.printStackTrace(); } 
                catch (RemoteException e) { e.printStackTrace(); }    
                
                currentIncident.newAlarm(newPeopleCount, newSensor); // Update Incident on new alarm.
            }
        }
        
        if (!foundMatchingIncident)  // This is a new Incident
        {
            currentIncident = new Incident(newSensor.getSchoolId(), newSchoolLocation, newPeopleCount, "AMBULANCE");
            VehicleItem newPrimaryVehicle = this.dispatchClosestEngine(newSensor, newSchoolLocation, true);
            currentIncident.setPimaryVehicle(newPrimaryVehicle);   
            this.incidents.add(currentIncident);  // add this Incident to the incident list.
        }
        
        // Dispatch Secondaries if needed.
        int totalSecondaryVehiclesNeeded = currentIncident.numberOfNewSocondaryVehiclesRequired();
        
        while (totalSecondaryVehiclesNeeded > 0)
        {
            VehicleItem newSecondaryVehicle = this.dispatchClosestEngine(newSensor, newSchoolLocation, false);
            currentIncident.addSecondaryVehicle(newSecondaryVehicle);
            totalSecondaryVehiclesNeeded--;
        }   
    }
    
    private Location convertToLocation(SchoolServiceStub.Location tempLoc)
    {
        Location location = new Location();
        location.setCordX(tempLoc.getCordX());
        location.setCordY(tempLoc.getCordY());
        return location;
    }
    
    
    // Needed these to properly generate web service. 
    public void normalcyMessage (String newSchool) 
    {
        super.normalcyMessage(newSchool);
    }
     
    public void vehicleUpdateReport (Location newLocation, String newVehicleID, int newVehicleStatus) 
    {  
        super.vehicleUpdateReport(newLocation, newVehicleID, newVehicleStatus);
    }

    public void vehicleAvailableForDispatch(String newVehicleID)
    {
        super.vehicleAvailableForDispatch(newVehicleID);
        
    }
    
    
    /**
     *  Internal use
     */
    protected int numberOfVehiclesRequired (int newTotalDetectors, int newPeopleCount) 
    {
        // Dispatch heuristic.
        int enginesNeeded = 1;
        
        if ( newTotalDetectors > 1 ) enginesNeeded++;
        if ( newTotalDetectors > 5 ) enginesNeeded++;
        if ( newPeopleCount > 100 ) enginesNeeded++;
        if ( newPeopleCount > 500 ) enginesNeeded++;
        
        return enginesNeeded;
    }

    /**
     *  Outgoing Messages
     */
    
    protected void dispatchPoolInit()
    {
        //iterate through dispatchPool, call UDDI get Vehicle details, update URL and Location.
            
        Iterator dispatchPoolIterator = this.dispatchPool.iterator();
        VehicleItem nextVehicle;
        int x = 1;
        
        while( dispatchPoolIterator.hasNext() )
        {
            nextVehicle = (VehicleItem)dispatchPoolIterator.next();
            
//            // get service URL for Vehicle
//            Map services = ServiceUtil.getServiceAccessPoints(nextVehicle.getVehicleID());
//            String serviceUrl = (String)services.keySet().iterator().next();
            String serviceName = nextVehicle.getVehicleID();
            String serviceUrl = "http://localhost:8080/"+serviceName.substring(0,serviceName.length()-1)+"WEB/services/"+serviceName;
            nextVehicle.setVehicleHandle(serviceUrl);
            
            // Ask VehicleItem to get updated location from Vehicle.
                
            nextVehicle.getLocationUpdateFromEngine();    
            this.updateGUI(nextVehicle);
            System.out.println("VehicleDispatchService.dispatchPoolInit: VehicleItem " + x++ + " init: serviceUrl: " + serviceUrl + ", vehicelID: " + nextVehicle.getVehicleID() + ", Location: " + nextVehicle.getCurrentLocation().toString() );
        }        
    }
    
    protected VehicleItem dispatchClosestEngine (Sensor newSensor, Location newLocation, boolean isPrimary) 
    {   
        Iterator dispatchPoolIterator = this.dispatchPool.iterator();
        VehicleItem nextVehicle = null;
        VehicleItem closestVehicle = (VehicleItem)dispatchPoolIterator.next();
        double currentClosestDistance = Double.MAX_VALUE;
        double nextDistance = Double.MAX_VALUE; 
        
        
        // Itterate through dispatchPool
        while( dispatchPoolIterator.hasNext() )
        {
            nextVehicle = (VehicleItem)dispatchPoolIterator.next();
            if ( !nextVehicle.isDispatched() )
            {
                // Determine if nextVehicel is closest.
                try
                {
                    nextDistance = newLocation.howFarIs(nextVehicle.getCurrentLocation());
                }
                catch (LocationMismatch e) { e.printStackTrace(); }
                
                if ( nextDistance < currentClosestDistance )
                {
                    closestVehicle = nextVehicle;
                    currentClosestDistance = nextDistance;
                }    
            }
            
            System.out.println(this.serviceContext.toString());
            System.out.println(closestVehicle);
        }
        
        closestVehicle.dispatch(newLocation, this.serviceContext.getAxisService().toString());
        
        this.updateGUI(closestVehicle);                   
        return closestVehicle;
    }
    

}

