package edu.swe622.fers.service;

import java.util.ArrayList;
import java.util.ResourceBundle;

import org.apache.axis2.context.ServiceContext;

import edu.swe622.fers.model.InfiniteLoopException;
import edu.swe622.fers.model.Location;
import edu.swe622.fers.model.LocationMismatch;
import edu.swe622.fers.model.NoPathException;
import edu.swe622.fers.model.Organization;
import edu.swe622.fers.model.PredefinedPaths;

/**
 * @author Naeem
 */
public abstract class VehicleService {
	public static final boolean IS_ON_MISSION = true; 
	public static final boolean IS_ON_RETURN = false; 
	
	public static final int DISPATCHED = 1;
    public static final int DISPATCHED_ON_WAY = 2;
    public static final int DISPATCHED_AT_LOCATION = 3;
    public static final int DISPATCHED_EXT_FIRE = 4;
    public static final int DISPATCHED_MISSION_COMPLETE = 5;
    public static final int RETURNING_TO_BASE = 6;
	/**
	 * Initializes the vehicle according the properties file
	 * @param name the name of Vehicle
	 */
	public void init(ServiceContext serviceContext) {
		this.name = (String) serviceContext.getAxisService().getName();
		
		ResourceBundle rb = ResourceBundle.getBundle("vehicleconfig");
		speed = Math.abs(Double.parseDouble(rb.getString(this.name + ".Speed")));
		sleeptime = Math.abs(Integer.parseInt(rb.getString(this.name + ".SleepTime")));
		
		if(Boolean.parseBoolean(rb.getString(this.name + ".HasPath"))) {
			int routeNumber = Integer.parseInt(rb.getString(this.name + ".RouteNumber"));
			
			if(routeNumber == 1) {
				path = PredefinedPaths.getPath1();
			} else if(routeNumber == 2) {
				path = PredefinedPaths.getPath2();
			} else if(routeNumber == 3) {
				path = PredefinedPaths.getPath3();
			} else if(routeNumber == 4) {
				path = PredefinedPaths.getPath4();
			}
						
			base = path.get(0).clone();
			current = path.get(0).clone();
			current.initPath(path, speed);
			
			System.out.println(name + " Has Path");
		}
		else {
			 base = new Location(Double.parseDouble(rb.getString(this.name + ".BaseX")), Double.parseDouble(rb.getString(this.name + ".BaseY"))); //$NON-NLS-1$ //$NON-NLS-2$
			 current = base.clone();

			 System.out.println(name + " Has Base");
		}
		
		System.out.println(name + " basics are initilized");
	}
		
	protected Location getCurrentLocation() {
		System.out.println(name + " is inside the getCurrentLocation");

		return current.clone();
	}

	protected String getName() {
		System.out.println(name + " is inside the getName");

		return name;
	}
		
	protected String getUrlByID(String id) {
//		ServiceUtil.setupProxy("juddi", "juddi");
		System.out.println("id:*******************************"+id);
//		Map organizationService =  ServiceUtil.getServiceAccessPoints(id);
//		System.out.println("Map: "+ organizationService);
//		String url = (String)organizationService.keySet().iterator().next();
		String url = "http://localhost:8080/"+id.substring(0,id.length()-1)+"WEB/services/"+id;
		return url;
	}		
	
	/**
	 * Dispatch <b>service</b>: Orders the vehicle to go to some place for some duty.
	 * This method should be extended for each kind of vehicle. 
	 * The implementors should make the busy flag false at the end of their extension.
	 * @param org The organization who has issued the command and should receive the reports
	 */
	protected void dispatchInitials(Organization org) {
		supervisor = org;
	}
	
	/**
	 * Return <b>service</b>: Orders the vehicle to its base station 
	 */
	protected void doReturn() {		
		goToLocation(base, IS_ON_RETURN);
	}

	/**
	 * Reporting vehicle status to the organization responsible for this Vehicle.
	 * @param report what to be sent over there
	 */
	protected abstract void reportBack(Location report, int status);

	/**
	 * Gets a location and goes there
	 * @param destination where it should go
	 * @param goToMission the direction
	 */
	protected synchronized void goToLocation(Location destination, boolean goToMission){
		System.out.println("Path: " + path);
		
		if(path != null && path.size() != 0)
			current.flipDestination(goToMission);
		

		if(mission instanceof Commuter)
			System.out.println("Commuter: " + mission);
		else
			System.out.println("Unknown: " + mission);
		
		if(mission != null)
			mission.doStop();
		
		System.out.println("Curren: " + current + " Destination: " + destination);
		
		if(goToMission == IS_ON_MISSION)
			mission = new Commuter(destination.clone(), DISPATCHED_ON_WAY);
		else
			mission = new Commuter(destination.clone(), RETURNING_TO_BASE);			
		
  		mission.start();
	}
	
	/**
	 * This is correspond to predefined paths 
	 */
	protected ArrayList<Location> path = null; 
	
	/**
	 * The organization to whom the vehicle should report back 
	 */
	protected Organization supervisor;	
			
	/**
	 * The location where the vehicle remains there in the idle times 
	 */
	protected Location base = null;
		
	/**
	 * The location where the vehicle currently exist 
	 */
	protected Location current = null;
	
	/**
	 * This thread corresponds to the current command which should be executed
	 */
	protected FERSThread mission = null;
	
	/**
	 * The speed of the vehicle in the notion of how much unit per one second
	 */
	protected double speed = 0.1;
		
	/**
	 * The sleep time to control the speed of simulation. Simulates 1 Second.
	 */
	protected int sleeptime = 100;
	
	protected String name = null;
	protected boolean geo = false;
	
	/**
	 * This class extends the basic Thread in the ways that are required for our project
	 * @author Naeem
	 */
	public abstract class FERSThread extends Thread {
		/**
		 * Provided for soft stop mechanism 
		 */
		public void doStop() {
			stop = true;
			this.interrupt();
		}
		
		/**
		 * Indicates if the soft stop command has been issued or not
		 */
		protected boolean stop = false;		
	}
	
	/**
	 * Goes to location to carry out the mission
	 * @author Naeem
	 */
	public class Commuter extends FERSThread {		
		public Commuter(Location destination, int status) {
			this.destination = destination;
			this.status = status;
		}
		
		public void run() {
			super.run();
			
			boolean arrived = false;
			
			try {
				do {
					
					reportBack(current, status);
					
					synchronized(VehicleService.this){
						if(!stop) {
							if(path != null)
								arrived = current.followPath();
							else if(destination != null)
								arrived = current.moveToward(destination, speed);
							else
								throw new NoPathException();
						}
					}
					Thread.sleep(sleeptime);
					
				}while(!arrived && !stop);
				
				reportBack(current, status);
				
				if(stop && !arrived)
					System.out.println("Received a new order and should abandon the previous command.");
			} catch (NoPathException e) {
				e.printStackTrace();
				System.out.println("There is no path defined."); //$NON-NLS-1$
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (InfiniteLoopException e) {
				e.printStackTrace();
				System.out.println("Cannot ever reach " + destination + " due to some cofiguration problems"); //$NON-NLS-1$ //$NON-NLS-2$
			} catch (LocationMismatch e) {
				e.printStackTrace();
				System.out.println("Two differnet kind of Location system exist in the system."); //$NON-NLS-1$
			}			
		}
		
		protected Location destination;
		protected int status;
	}
}
