package edu.swe622.fers.service;

import java.util.HashSet;
import java.util.Iterator;

import edu.swe622.fers.model.Location;
import edu.swe622.fers.model.Sensor;


public class Incident 
{
    String schoolID;
    Location schoolLocation;
    int peopleCount;
    //String smokeDetectorHandles;
    //String sprinklerHandles;
    String incidentType;
    
    int alarmCount;
    VehicleItem primaryVehicle;
    HashSet secondaryDispatchedVehicles;
    
    
    public Incident(String newSchoolID, Location newSchoolLocation, int newPeopleCount, String newIncidentType )
    {
       this.schoolID = newSchoolID;
       this.schoolLocation = new Location(newSchoolLocation);
       this.peopleCount = newPeopleCount;
       this.incidentType = newIncidentType;  // FIRE or AMBULANCE
       
       this.alarmCount = 1;
       this.secondaryDispatchedVehicles = new HashSet();
    }
    
    public void newAlarm(int newPeopleCount, Sensor newSensor)
    {
        if (newPeopleCount > this.peopleCount) this.peopleCount = newPeopleCount;
        this.alarmCount++;
        
        // If incident type is fire, update the primary with new sensor info.
        if ( this.incidentType.endsWith("FIRE") )
            this.primaryVehicle.addSensorToPrimaryVehicle(newSensor); 
    }
    
    public int numberOfNewSocondaryVehiclesRequired() 
    {
        return this.totalSecondaryVehiclesNeeded() - this.secondaryDispatchedVehicles.size();
    }
    
    public void setPimaryVehicle(VehicleItem newPrimaryVehicle)
    {
        this.primaryVehicle = newPrimaryVehicle;
    }
    
    public void addSecondaryVehicle(VehicleItem newSecondaryVehicle)
    {
        this.secondaryDispatchedVehicles.add(newSecondaryVehicle);
    }
    
    public void incidentRecall()
    {
        if ( this.incidentType.endsWith("AMBULANCE") )
            this.primaryVehicle.recallAmbulance();
        
        if ( this.incidentType.endsWith("FIRE") )
            this.primaryVehicle.recallEngine();
        
        Iterator secondaryVehicleIterator = this.secondaryDispatchedVehicles.iterator();
        
        VehicleItem nextVehicle;
        while( secondaryVehicleIterator.hasNext() )
        {
            nextVehicle = (VehicleItem)secondaryVehicleIterator.next();
            if ( this.incidentType.endsWith("AMBULANCE") )
                nextVehicle.recallAmbulance();
            if ( this.incidentType.endsWith("FIRE") )
                nextVehicle.recallEngine();
        }
    }
    
    // Dispatch heuristic
    private int totalSecondaryVehiclesNeeded()
    {
        int total = 0;
        
        if (this.alarmCount > 1) total++;
        if (this.alarmCount > 5) total++;
        if (this.peopleCount > 100) total++;
        if (this.peopleCount > 400) total++;
        if (this.peopleCount > 1000) total++;
        
        return total;
    }
    
    public String getSchoolID()
    {
        return this.schoolID;
    }

     
    public boolean equals (Object otherObject) 
    {
        if ( otherObject == this ) return true;
        if ( otherObject == null ) return false;
        if ( otherObject.getClass() != this.getClass() ) return false;
        
        Incident otherIncident = (Incident) otherObject;
        
        if ( this.schoolID.equals(otherIncident.schoolID) ) return true;
        
        return false;
    }

    public int hashCode () 
    {
        return this.schoolID.hashCode();
    }
    
    public boolean repOK()
    {
        if (this.alarmCount < 1) return false;
        if (this.peopleCount < 0) return false;
        if (this.primaryVehicle == null) return false;
        if (this.schoolID == null) return false;
        if (this.schoolID.equals("")) return false;
        if (this.schoolLocation == null) return false;
        if (this.secondaryDispatchedVehicles == null) return false;
        
        return true;
    }
    

}
