package edu.swe622.fers.util;

import java.util.Vector;

import org.uddi4j.UDDIException;
import org.uddi4j.datatype.Description;
import org.uddi4j.datatype.Name;
import org.uddi4j.datatype.business.BusinessEntity;
import org.uddi4j.datatype.business.Contact;
import org.uddi4j.datatype.business.Contacts;
import org.uddi4j.datatype.business.Email;
import org.uddi4j.response.BusinessDetail;
import org.uddi4j.response.BusinessInfo;
import org.uddi4j.response.BusinessList;
import org.uddi4j.response.DispositionReport;
import org.uddi4j.transport.TransportException;
import org.uddi4j.util.CategoryBag;
import org.uddi4j.util.KeyedReference;

public class BusinessUtilities extends Utilities 
{
	/**
	* Display business information of a BusinessEntity
	* @param businessKey The businessKey of the BusinessEntity who's detail is to be displayed
	*/
	public static void showBusinessDetail(String businessKey)
	{
		BusinessDetail businessDetail = null;
		try
		{
			// Get the business detail from the registry based on the key
			businessDetail = proxy.get_businessDetail(businessKey);
		}
		catch(TransportException t)
		{
			System.out.println("Transport Exception: " + t.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
		
		// Get the first business entity returned
		Vector businessEntityVector = businessDetail.getBusinessEntityVector();
		
		if(businessEntityVector.size() == 0)
		{
			System.out.println("Could not locate business for businessKey: " + businessKey);
		}
		
		BusinessEntity businessEntity = (BusinessEntity) businessEntityVector.get(0);
		
		// Display business name
		Vector businessNames = businessEntity.getNameVector();
		
		for (int i = 0; i < businessNames.size(); i++) 
		{
			Name businessName = (Name) businessNames.get(i);
			System.out.println("Business Name : " + businessName.getText() + "(Language: " + businessName.getLang() +")");
		}
		
		// Display descriptions
		Vector descriptions = businessEntity.getDescriptionVector();
		
		for (int i = 0; i < descriptions.size(); i++) 
		{
			Description description = (Description) descriptions.get(i);
			System.out.println("Description:\n" + description.getText());
		}
		
		// Display contacts
		System.out.println("Contacts for this Business");
		Contacts contacts = businessEntity.getContacts();
		
		for (int i = 0; i < contacts.size(); i++) 
		{
			Contact contact = (Contact) contacts.get(i);
			System.out.println("Name: " + contact.getPersonName().getText() + " Role: " + contact.getUseType());
			Vector emailVector = contact.getEmailVector();
			for (int j = 0; j < emailVector.size(); j++) 
			{
				Email email = (Email) emailVector.get(j);
				System.out.println("Email: " + email.getText() + "(" + email.getUseType() + ")");
			}
		}
	} // end showBusinessDetail
	
	/**
	* Locate a Business by name
	* @param busName The business name used to search for a business
	* @return Vector: This method returns a Vector of Strings.  Each String represents the businessKey of a business matching the query
	*/
	public static Vector findBusinessByName(String busName)
	{
		Vector names = new Vector();
		Vector businessKeyStringVector = new Vector();
		Vector businessInfoVector = null;
		Name name = new Name();
	
		System.out.println("busName: " + busName);                        
		name.setText(busName);
		names.add(name);
	
		try
		{
			BusinessList businessList = proxy.find_business(names, null, null, null, null,null, 0);
			businessInfoVector = businessList.getBusinessInfos().getBusinessInfoVector();//Now display the amount of business found, and for each one, get the BusinessKey, the BusinessName and the amount of Services that Business has	
			System.out.println("There are " + businessList.getBusinessInfos().size() + " matching Businesses in this registry");	
		
			for(int i=0; i < businessList.getBusinessInfos().size(); i++)
			{
				System.out.println("BusinessKey = " + ((BusinessInfo)businessInfoVector.get(i)).getBusinessKey());                        
				businessKeyStringVector.addElement(((BusinessInfo)businessInfoVector.get(i)).getBusinessKey());
			}
		}
		catch(TransportException te)
		{
			System.out.println("Transport Exception: " + te.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
	
		if(businessInfoVector.size() <= 0)
		{
			System.out.println("No Businesses found for Name: " + busName);
		}
		
		return businessKeyStringVector;
	} // end findBusinessByName
	
	/**
	* Locate a Business by key
	* @param businessKey The business key used to search for a business
	* @return BusinessList: This function returns a BusinessList on success.  In the event that no matches were located for the specified criteria, a BusinessList structure with zero BusinessInfo structures is returned.
	*/
	public static BusinessList findBusinessByKey(String businessKey)
	{
		BusinessList businessList = null;
		KeyedReference reference = new KeyedReference();
		reference.setTModelKey(businessKey);
		CategoryBag categoryBag = new CategoryBag();
		categoryBag.add(reference);

		try
		{
			businessList = proxy.find_business(null, null, null, categoryBag, null, null, 0);  //find business based on Key
		}
		catch(TransportException t)
		{
			System.out.println("Transport Exception: " + t.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
		
		return businessList;
	} // end findBusinessByKey

	/**
	* Delete a Business by name
	* @param busName Business name used to find and delete a business
	*/
	public static void deleteBusinessByName(String busName)
	{
		Vector businessInfoKeysVector = null;
		
		try
		{
			if(authInfo == null)
			{
				authInfo = getAuthorization(); 
			}
		
			businessInfoKeysVector = findBusinessByName(busName);
			
			for(int i = 0; i < businessInfoKeysVector.size(); i++)
			{
				System.out.println("Preparing to delete Business key " + businessInfoKeysVector.elementAt(i).toString());
				DispositionReport report = proxy.delete_business(authInfo, businessInfoKeysVector.elementAt(i).toString());
				
				if(report.success())
				{
					System.out.println("Business deleted successfully");
				}
				else
				{
					handleDispositionReport(report);
				}
			} 
		}
		catch(TransportException te)
		{
			System.out.println("Transport Exception: " + te.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
	} // end deleteBusinessByName
	
	/**
	* Delete a Business by key
	* @param businessKey A Business key used to find and delete a business
	*/
	public static void deleteBusinessByKey(String businessKey)
	{
		try
		{
			if(authInfo == null)
			{
				authInfo = getAuthorization(); 
			}

			System.out.println("Preparing to delete Business key " + businessKey);
			DispositionReport report = proxy.delete_business(authInfo, businessKey);
		
			if(report.success())
			{
				System.out.println("Business deleted successfully");
			}
			else
			{
				System.out.println("Problem encountered during deleting Business");
				handleDispositionReport(report);
			}
		}
		catch(TransportException te)
		{
			System.out.println("Transport Exception: " + te.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
	} // end deleteBusinessByKey
	
	/**
	* Publish a Business
	* @param busName The name of the business to be published
	* @return String: This method returns a String representing the key of the newly published Business
	*/
	public static String publishBusiness(String busName)
	{
		Vector businessNames = new Vector();
		Vector businessEntities = new Vector();
		Vector businessDetail = null;
		
		try 
		{
			if(authInfo == null)
			{
				authInfo = getAuthorization(); 
			}
		
			BusinessEntity businessEntity = new BusinessEntity();
			businessEntity.setBusinessKey("");
			Name businessName = new Name();
			businessName.setText(busName);
			businessNames.add(businessName);
		
			businessEntity.setNameVector(businessNames);
		
			businessEntities.add(businessEntity);
			businessDetail = (proxy.save_business(authInfo, businessEntities)).getBusinessEntityVector();
			System.out.println("Business Key: " + ((BusinessEntity)businessDetail.firstElement()).getBusinessKey());
			System.out.println("Business " + ((BusinessEntity)businessDetail.firstElement()).getDefaultNameString() + " saved successfully.");
		
			return ((BusinessEntity)businessDetail.firstElement()).getBusinessKey();
		}
		catch (Exception e)
		{
			System.out.println("Attempting to publish the business entity to the UDDI: " + e.toString());
			return null;	
		}
	} // end publishBusiness
} // end BusinessUtilities
