package edu.swe622.fers.util;


import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.uddi4j.UDDIException;
import org.uddi4j.datatype.Description;
import org.uddi4j.datatype.Name;
import org.uddi4j.datatype.binding.AccessPoint;
import org.uddi4j.datatype.binding.BindingTemplate;
import org.uddi4j.datatype.binding.BindingTemplates;
import org.uddi4j.datatype.binding.TModelInstanceDetails;
import org.uddi4j.datatype.binding.TModelInstanceInfo;
import org.uddi4j.datatype.service.BusinessService;
import org.uddi4j.response.DispositionReport;
import org.uddi4j.response.ServiceDetail;
import org.uddi4j.response.ServiceInfo;
import org.uddi4j.response.ServiceList;
import org.uddi4j.transport.TransportException;
import org.uddi4j.util.CategoryBag;
import org.uddi4j.util.KeyedReference;

@SuppressWarnings("unchecked")
public class ServiceUtil extends Utilities
{
	/**
	* Locate a Service by name
	* @param serviceNames A vector of Name objects
	* @return Map: This method returns a Map of service and business key pairs 
	*/
	public static Map findServiceByName(Vector serviceNames)
	{
		Vector serviceInfoVector = null;
		
		try
		{
			ServiceList myServiceList =	proxy.find_service(null, serviceNames, null, null, null, 0);
			System.out.println("myServiceListL"+myServiceList);
			serviceInfoVector =	myServiceList.getServiceInfos().getServiceInfoVector();
		}
		catch(TransportException t)
		{
			System.out.println("Transport Exception: " + t.toString());
			t.printStackTrace();
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
			ue.printStackTrace();
		}
	
		if(serviceInfoVector.size() == 0)
		{
			System.out.println("No Services found.");
		}
		
		Map serviceKeysMap = new HashMap();
		
		for (int i = 0; i < serviceInfoVector.size(); i++) 
		{
			ServiceInfo serviceInfo = (ServiceInfo) serviceInfoVector.get(i);
			String serviceKey = serviceInfo.getServiceKey();
			String businessKey = serviceInfo.getBusinessKey();
			serviceKeysMap.put(serviceKey, businessKey);
			System.out.println("BusinessKey = " + businessKey);
			System.out.println("ServiceKey = " + serviceKey);                        
		}
	
		return serviceKeysMap;
	} // end findServiceByName

	/**
	* Locate a Service by key
	* @param serviceKey A key used to search for a Service
	* @return ServiceList: This method returns a ServiceList of ServiceInfos objects
	*/
	public static ServiceList findServiceByKey(String serviceKey)
	{
		ServiceList serviceList = null;
		KeyedReference reference = new KeyedReference();
		reference.setTModelKey(serviceKey);
		CategoryBag categoryBag = new CategoryBag();
		categoryBag.add(reference);

		try
		{
			serviceList = proxy.find_service(null, null, categoryBag, null, null, 0);  //find service based on Key
		}
		catch(TransportException t)
		{
			System.out.println("Transport Exception: " + t.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
		
		return serviceList;
	} // end findServiceByKey

	/**
	* Delete a Service by name
	* @param serviceName Service name used to find and delete a Service
	*/
	public static void deleteServiceByName(String serviceName)
	{
		//Locate services for the Name specified
		Name name = new Name();
		Vector serviceNames = new Vector();
		Vector serviceKeyStrings = new Vector();
	
		try
		{
			if(authInfo == null)
			{
				authInfo = getAuthorization(); 
			}
		
			name.setText(serviceName);
			serviceNames.addElement(name);
			serviceMap = findServiceByName(serviceNames);
		
			//Get a set of service keys
			Set serviceSet = serviceMap.keySet();
			Iterator it = serviceSet.iterator();
		
			while(it.hasNext())
			{
				String s = (String)it.next();
				serviceKeyStrings.addElement(s);
				System.out.println("Added " + s + " to the vector.");
			}
		
			DispositionReport report = proxy.delete_service(authInfo, serviceKeyStrings);
		
			if(report.success())
			{
				System.out.println("Service deleted successfully");
			}
			else
			{
				System.out.println("Problem encountered during deleting Service");
				handleDispositionReport(report);
			}
		}
		catch(TransportException te)
		{
			System.out.println("Transport Exception: " + te.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
	} // end deleteServiceByName
	
	/**
	* Delete a Service by key
	* @param serviceKey Service key used to find and delete a Service
	*/
	public static void deleteServiceByKey(String serviceKey)
	{
		try
		{
			if(authInfo == null)
			{
				authInfo = getAuthorization(); 
			}

			System.out.println("Preparing to delete Service key " + serviceKey);
			DispositionReport report = proxy.delete_service(authInfo, serviceKey);
		
			if(report.success())
			{
				System.out.println("Service deleted successfully");
			}
			else
			{
				System.out.println("Problem encountered during deleting Service");
				handleDispositionReport(report);
			}
		}
		catch(TransportException te)
		{
			System.out.println("Transport Exception: " + te.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
	} // end deleteServiceByKey
	
	/**
	* Publish a Service
	* @param serviceName The name of the Service to be published
	* @param businessName The Business name used to associate the service to.  If Business name does not exist in the registry, a Business entry will be created.
	* @param tModelName The TModel that the service implements.  If TModel is not found in the registry and the TModelOverviewDoc parameter is not null, a TModel entry will be created.
	* @param tModelOverviewDoc Required only if the TModel name provided in the tModelName parameter is not found in the registry.  This parameter expects a URL-String representation of the WSDL address.  EX: http://localhost:9080/MyRouter/Test.wsdl
	* @param accessPointStr
	* @param description Optional Service description.
	* @return String: This method returns a String representing the key of the newly published Service
	*/
	public static String publishService(String serviceName, String busName, String tModelName, String tModelOverviewDoc, String accessPointStr, String description)
	{
		BusinessService businessService = new BusinessService();
		Vector businessInfoKeysVector = null;
		ServiceDetail serviceDetails = null;
		
		if(serviceName == null || serviceName.trim().equals(""))
		{
			System.out.println("Service name must not be NULL!");
			return null;
		}
		else if(busName == null || busName.trim().equals(""))
		{
			System.out.println("Business name must not be NULL!");
			return null;
		}
		else if(tModelName == null || tModelName.trim().equals(""))
		{
			System.out.println("TModel name must not be NULL!");
			return null;
		}
		else if(accessPointStr == null || accessPointStr.trim().equals(""))
		{
			System.out.println("TModel accessPoint name must not be NULL!");
			return null;
		}
		else
		{
			try
			{
				Vector serviceNames = new Vector();
				Vector descriptions = new Vector();
				TModelInstanceInfo tModelInstanceInfo = null;
				Vector tModelInstanceInfoList = new Vector();
				Vector tModelKeys = null;
				String businessKey = null;
				String tModelKey = null;
				
				businessInfoKeysVector = BusinessUtilities.findBusinessByName(busName);
				
				if(businessInfoKeysVector.size() == 0)
				{
					businessKey = BusinessUtilities.publishBusiness(busName);
				}
				else
				{
					businessKey = businessInfoKeysVector.firstElement().toString();
				}
			
				if(businessKey == null || businessKey.trim().equals(""))
				{
					System.out.println("No suitable Business found.\nService " + serviceName + " was NOT published.");
					return null;		
				}
				
				serviceNames.add(new Name(serviceName));
				businessService.setBusinessKey(businessKey);
				businessService.setNameVector(serviceNames);
				businessService.setServiceKey(""); //Create a new service key
				descriptions.add(new Description(description)); //Add Description
				businessService.setDescriptionVector(descriptions);
			
				//Adding an access point
				BindingTemplate bindingTemplate = new BindingTemplate();
				bindingTemplate.setBindingKey("");
				AccessPoint accessPoint = new AccessPoint();
				accessPoint.setURLType("http");
				accessPoint.setText(accessPointStr);
				bindingTemplate.setAccessPoint(accessPoint);
			
				//Adding tModel instance data
				tModelKeys = ModelUtilities.findTModelByName(tModelName);
				
				if(tModelKeys.size() == 0)  //TModel was not found.  Try to register a new instance
				{
					if(tModelOverviewDoc == null || tModelOverviewDoc.trim().equals(""))
					{
						System.out.println("tModelOverviewDoc must be provided.\nService " + serviceName + " was NOT published.");
						return null;
					}
					else
					{
						tModelKey = ModelUtilities.publishTModel(tModelName,tModelOverviewDoc);
					}
				}
				else  //Found TModel.  Get key for adding to service.
				{
					tModelKey = tModelKeys.firstElement().toString();
				}
			
				if(tModelKey == null || tModelKey.trim().equals(""))
				{
					System.out.println("No suitable TModel found.\nService " + serviceName + " was NOT published.");
					return null;		
				}
				
				tModelInstanceInfo = new TModelInstanceInfo(tModelKey);
				tModelInstanceInfoList.addElement(tModelInstanceInfo);
				TModelInstanceDetails tModelInstanceDetails = new TModelInstanceDetails();
				tModelInstanceDetails.setTModelInstanceInfoVector(tModelInstanceInfoList);
				bindingTemplate.setTModelInstanceDetails(tModelInstanceDetails);
				BindingTemplates bindingTemplates = new BindingTemplates();
				bindingTemplates.add(bindingTemplate);
				businessService.setBindingTemplates(bindingTemplates);
			
				//Add category information for this businessEntity
				KeyedReference reference_US = new KeyedReference();
				reference_US.setKeyName("Specification for a Web Service described in WSDL");
				reference_US.setKeyValue("wsdlSpec");
				reference_US.setTModelKey("UUID:C1ACF26D-9672-4404-9D70-39B756E62AB4");
				CategoryBag categoryBag = new CategoryBag();
				categoryBag.add(reference_US);
				businessService.setCategoryBag(categoryBag);
				Vector businessServices = new Vector();
				businessServices.add(businessService);
	
				if(authInfo == null)
				{
					authInfo = getAuthorization(); 
				}
				
				serviceDetails = proxy.save_service(authInfo,businessServices);
				System.out.println("Service Key: " + ((BusinessService)serviceDetails.getBusinessServiceVector().firstElement()).getBusinessKey());
				System.out.println("Service " + ((BusinessService)serviceDetails.getBusinessServiceVector().firstElement()).getDefaultNameString() + " published successfully.");
			}
			catch(UDDIException ue)
			{
				System.out.println("UDDI Exception: " + ue.toString());
				return null;
			}
			catch(Exception e)
			{
				e.printStackTrace();
				return null;
			}
			
			return ((BusinessService)serviceDetails.getBusinessServiceVector().firstElement()).getBusinessKey();
		}
	} // end publishService
	
	private static Map getServiceURL(Set serviceKeySet)
	{
		Vector serviceKeys = new Vector(serviceKeySet);
		return getServiceURL(serviceKeys);
	}

	/**
	* Get the URL of the actual service, given the UUID of a BusinessService
	* @param Vector ServiceKeys of the BusinessService(s)
	* @return Map: Map with URL-serviceKey pairs
	*/
	private static Map getServiceURL(Vector serviceKeys)
	{
		Vector businessServices = null;
		Map urlMap = new HashMap();
	
		try
		{
			ServiceDetail serviceDetail = proxy.get_serviceDetail(serviceKeys);
			businessServices = serviceDetail.getBusinessServiceVector();
		}
		catch(TransportException t)
		{
			System.out.println("Transport Exception: " + t.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}

		if(businessServices.size() == 0)
		{
			System.out.println("Failed to find business services");
		}
	
		for(int j = 0; j < businessServices.size(); j++) 
		{
			BusinessService businessService = (BusinessService) businessServices.get(j);
			String serviceKey = businessService.getServiceKey();
			Vector bindingTemplateVector=businessService.getBindingTemplates().getBindingTemplateVector();
			BindingTemplate bindingTemplate = null;
		
			for(int i = 0; i < bindingTemplateVector.size(); i++) 
			{
				// find the binding template with an http access point
				bindingTemplate = (BindingTemplate)bindingTemplateVector.elementAt(i);
				AccessPoint accessPoint = bindingTemplate.getAccessPoint();
			
				if(accessPoint.getURLType().equals("http")) 
				{
					System.out.println("accessPoint.getText(): " + accessPoint.getText());
					urlMap.put(accessPoint.getText(),serviceKey);
				}
			} // end for
		} // end for
		return urlMap;
	} // end getServiceURL
	
	/**
	* Get the ServiceURLs for all available services
	* @return Map: This method returns a HashMap containing URL-serviceKey pairs
	*/
	public static synchronized Map getServiceAccessPoints(String serviceName)
	{
		Map urls = new HashMap();
		Vector serviceNameVector = new Vector();
		Name name = new Name();
		
		//Locate services for the Technical Model specified
		name.setText(serviceName);
		serviceNameVector.addElement(name);
		serviceMap = findServiceByName(serviceNameVector);
	
		if(serviceMap != null)
		{
			//Get a set of service keys
			Set serviceSet = serviceMap.keySet();
			urls =  getServiceURL(serviceSet);
			System.out.println("Number of services found: " + serviceMap.size());
		}
		
		return urls;
	}
} // end ServiceUtilities

