package edu.swe622.fers.service;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.ResourceBundle;

import org.apache.axis.AxisFault;
import org.apache.axis2.context.ServiceContext;

import edu.swe622.fers.model.Location;
import edu.swe622.fers.model.Organization;
import edu.swe622.fers.model.Sensor;

/**
 * @author Naeem
 */
public class FireFightingEngineService extends VehicleService {
		
	protected ArrayList<Sensor> smokeDetectors = new ArrayList<Sensor>();

	/**
	 * @param dst Where this vehicle is going for duty
	 * @param isPrimary if it is the commander in the mission
	 * @param smokeDetectors smoke detectors that should be checked by this FFE
	 * @see Vehicle.dispatch()
	 */
	public void dispatch(Organization org, Location dst, boolean isPrimary, Sensor smokeDetector) {
		System.out.println(name + " is inside the dispatch");

		super.dispatchInitials(org);
		
		smokeDetectors.add(smokeDetector);
		
		if(isPrimary)
			fightFire(dst);
		else
			goToLocation(dst, IS_ON_MISSION);		
	}

	/**
	 * Reporting vehicle status to the organization responsible for this Vehicle.
	 * @param report what to be sent over there
	 */
	protected void reportBack(Location report, int status) {
		EngineDispatchServiceStub orgstub;
		
		try {
			orgstub = new EngineDispatchServiceStub(getUrlByID(supervisor.getUri()));
			
			EngineDispatchServiceStub.VehicleUpdateReport myreport = new EngineDispatchServiceStub.VehicleUpdateReport();
			myreport.setNewLocation(new EngineDispatchServiceStub.Location());
			myreport.getNewLocation().setCordX(report.getCordX());
			myreport.getNewLocation().setCordY(report.getCordY());
			
			myreport.setNewVehicleID(name);
			myreport.setNewVehicleStatus(status);
			
			orgstub.vehicleUpdateReport(myreport);
		} catch (AxisFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
		supervisor.getReport(report, name, status);
	}
	
	@Override
	public void doReturn() {
		if(supervisor == null) {
			System.out.println(name + " is inside the doReturn But no supervisor");
			return;
		}
		
		System.out.println(name + " is inside the doReturn");

		EngineDispatchServiceStub orgstub;

		try {
			orgstub = new EngineDispatchServiceStub(getUrlByID(supervisor.getUri()));
			
			EngineDispatchServiceStub.VehicleAvailableForDispatch myreport = new EngineDispatchServiceStub.VehicleAvailableForDispatch();
			myreport.setNewVehicleID(name);
			
			orgstub.vehicleAvailableForDispatch(myreport);
		} catch (AxisFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}		
		
		supervisor.released(name);

		super.doReturn();
	}

	@Override
	public Location getCurrentLocation() {
		return super.getCurrentLocation();
	}

	@Override
	public String getName() {
		return super.getName();
	}
	
	@Override
	public void init(ServiceContext serviceContext) {
		super.init(serviceContext);
		
		ResourceBundle rb = ResourceBundle.getBundle("vehicleconfig");
		fireExtinguishTime = Integer.parseInt(rb.getString(this.name + ".FireExtinguishTime"));
		
		System.out.println(name + " is completely initilized");
	}

	/**
	 * Add newly triggered sensors
	 * @param newSD the sensor to be added
	 */
	public void addSensor(Sensor newSD) {
		System.out.println(name + " is inside the addSensor");

		smokeDetectors.add(newSD);
	}

	/**
	 * Fights with the fire if it finds real fire
	 * @param destination where the fire is reported
	 * @param smokeDetectors a set of related smoke detectors in that particular place
	 * @param fireSprinklers a set of related sprinklers in that particular place
	 */
	private synchronized void fightFire(Location destination) {
		if(mission != null)
			mission.doStop();
		
		mission = new ExtinguishFire(destination.clone());
		mission.start();
	}

	/**
	 * How long it takes for the FFE to extinguish the fire 
	 */
	protected int fireExtinguishTime = 12;
	
	/**
	 * Goes to location to carry out the mission and afterwards extinguishes the potential fire 
	 * @author Naeem
	 */
	class ExtinguishFire extends Commuter {
		public ExtinguishFire(Location destination) {
			super(destination, VehicleService.DISPATCHED_ON_WAY);
		}
		
		@Override
		public void run() {
			super.run();

			System.out.println(name + "Starting probing the area for fire");
			
			try {
				if (detectFire()) {
					System.out.println(name + "Trying to turn off sensors");
					
					//TODO Check
					for (Iterator<Sensor> it = smokeDetectors.iterator(); it.hasNext();) {
						Sensor sd = it.next();
						SchoolServiceStub.Location sensorloc = new SchoolServiceStub.Location();
						sensorloc.setCordX(sd.getLocation().getCordX());
						sensorloc.setCordY(sd.getLocation().getCordY());						
	
			    		SchoolServiceStub schoolstub = new SchoolServiceStub(getUrlByID(sd.getSchoolId()));
			    		SchoolServiceStub.GetNearestSprinkler gns = new SchoolServiceStub.GetNearestSprinkler();						
						gns.setLocation(sensorloc);						
	
						String sprinklerid = schoolstub.getNearestSprinkler(gns).get_return().getSensorId();
						
			    		SensorServiceStub sprinklerstub = new SensorServiceStub(getUrlByID(sprinklerid));
			    		sprinklerstub.turnOffSensor();			    		
					}
					
					extinguishFire();
				}
				
				System.out.println(name + "Trying to turn of All the sensors");
				
				//TODO Check
				SchoolServiceStub sss = new SchoolServiceStub(getUrlByID(smokeDetectors.get(0).getSchoolId()));
				sss.turnOffAllSmokeDetectors();
			} catch (AxisFault e) {
				e.printStackTrace();
			} catch (RemoteException e) {
				e.printStackTrace();
			}
			
			System.out.println(name + ": Mission accomplished.");
		}

		/**
		 * Extinguishes fire and reports back in every time interval  
		 */
		private void extinguishFire() {
			System.out.println(name + " Trying to Extinguish Fire");
			int i = 0;
			
			for (; i < fireExtinguishTime && !stop; i++) {
				try {
					reportBack(current, VehicleService.DISPATCHED_EXT_FIRE);
					Thread.sleep(sleeptime);
				} catch (InterruptedException e) {
					e.printStackTrace();
					System.out.println(name + " Received a new order.");
				}
			}
			
			reportBack(current, VehicleService.DISPATCHED_MISSION_COMPLETE);			
			
			if(i == fireExtinguishTime)
				System.out.println(name + " Fire Extinguished");
			else
				System.out.println("Received a new order and should abandon the previous command.");
		}
		
		/**
		 * If at least one of the sensors is detected the real fire it returns true. Up to that point it turns the other detectors off
		 * @param smokeSensors list of sensors which it should check them
		 * @return true: fire, false: false positive
		 * @throws RemoteException 
		 * @throws AxisFault 
		 */
		private boolean detectFire() throws AxisFault, RemoteException {
			System.out.println(name + " Checking if there is fire fire");
			
			if(smokeDetectors.size() > 1)
				return true;
			else {//Only one of them
				//TODO Check
				SchoolServiceStub sss = new SchoolServiceStub(getUrlByID(smokeDetectors.get(0).getSchoolId()));
				return sss.isFire().get_return();
			}
		}
	}
}
